require 'train'
require 'pg'

module TrainPlugins
  module Pgsql
    class Connection < Train::Plugins::Transport::BaseConnection
      def initialize(options)
        super(options)
        enable_cache(:command)
      end

      def platform
        Train::Platforms.name('pgsql').in_family('sql')
        force_platform!('pgsql',
                        release: TrainPlugins::Pgsql::VERSION)
      end

      def close
        return if @connection.nil?

        logger.info format('[Pgsql] Closed connection to %s:%d', @options[:host], @options[:port])
        @connection.finish
      ensure
        @connection = nil
      end

      def uri
        "pgsql://#{options[:user]}@#{@options[:host]}:#{@options[:port]}/#{@options[:database]}"
      end

      def run_command_via_connection(query)
        stdout = ''
        stderr = ''
        exit_status = 0

        logger.debug format('[Pgsql] Sending command (%s) to %s:%d', query, @options[:host], @options[:port])
        begin
          connection.exec(query) do |result|
            stdout = {
              'fields' => result.fields,
              'values' => result.values
             }
            # stdout = result.values
          end
        rescue PG::Error => e
          stderr = e.message
        end
        CommandResult.new(stdout, stderr, exit_status)
      end

      def connection
        @connection ||= connect
      end

      def connect
        logger.info format('[Pgsql] Opening connection to %s:%d', @options[:host], @options[:port])

        @connection = PG::Connection.new(
          host: @options[:host],
          port: @options[:port],
          user: @options[:user],
          password: @options[:password],
          dbname: @options[:database]
        )

        @connection
      end
    end
  end
end
